@@ -1,6 +1,6 @@
 package Module::Build::Base;
 
-# $Id: Base.pm,v 1.38 2002/11/14 07:35:38 kwilliams Exp $
+# $Id: Base.pm,v 1.42 2002/11/20 08:58:46 kwilliams Exp $
 
 use strict;
 use Config;
@@ -135,7 +135,6 @@ sub resume {
   my $self = bless {@_}, $package;
   
   $self->read_config;
-  $self->{new_cleanup} = {};
   return $self;
 }
 
@@ -253,24 +252,22 @@ sub version_from_file {
 
 sub add_to_cleanup {
   my $self = shift;
-  @{$self->{new_cleanup}}{@_} = ();
-}
-
-sub write_cleanup {
-  my ($self) = @_;
-  return unless %{$self->{new_cleanup}};  # no new files
+  my @need_to_write = grep {!exists $self->{cleanup}{$_}} @_;
+  return unless @need_to_write;
   
-  # Merge the new parameters into the old
-  @{ $self->{cleanup} }{ keys %{ $self->{new_cleanup} } } = ();
+  if ( my $file = $self->config_file('cleanup') ) {
+    if ( !$self->{cleanup_fh} ) {
+      open $self->{cleanup_fh}, ">> $file" or die "Can't append to $file: $!";
+    }
+    print {$self->{cleanup_fh}} "$_\n" foreach @need_to_write;
+  }
   
-  # Write to the cleanup file
-  my $cleanup_file = $self->config_file('cleanup');
-  open my $fh, ">$cleanup_file" or die "Can't write '$cleanup_file': $!";
-  print $fh map {"$_\n"} sort keys %{$self->{cleanup}};
+  @{$self->{cleanup}}{ @need_to_write } = ();
 }
 
 sub config_file {
   my $self = shift;
+  return unless -d $self->{properties}{config_dir};
   return File::Spec->catfile($self->{properties}{config_dir}, @_);
 }
 
@@ -312,9 +309,6 @@ sub write_config {
   my @items = qw(requires build_requires conflicts recommends);
   print $fh Data::Dumper::Dumper( { map {$_,$self->{properties}{$_}} @items } );
   close $fh;
-
-  $self->add_to_cleanup('blib');
-  $self->write_cleanup;
 }
 
 sub prereq_failures {
@@ -474,11 +468,7 @@ my \$build = resume $build_package (
     build_script => '$build_script',
   },
 );
-eval {\$build->dispatch};
-my \$err = \$@;
-\$build->write_cleanup;  # Always write, even if error occurs
-die \$err if \$err;
-
+\$build->dispatch;
 EOF
 }
 
@@ -658,7 +648,7 @@ sub ACTION_build {
   if ($self->{properties}{c_source}) {
     $self->process_PL_files($self->{properties}{c_source});
     
-    my $files = $self->rscan_dir($self->{properties}{c_source}, qr{\.c$});
+    my $files = $self->rscan_dir($self->{properties}{c_source}, qr{\.c(pp)?$});
     
     push @{$self->{include_dirs}}, $self->{properties}{c_source};
 
@@ -672,6 +662,7 @@ sub ACTION_build {
 
   $self->process_PL_files('lib');
 
+  $self->add_to_cleanup('blib');
   my $files = $self->rscan_dir('lib', qr{\.(pm|pod|xs)$});
   $self->lib_to_blib($files, 'blib');
 }
@@ -756,7 +747,7 @@ sub ACTION_distclean {
 sub ACTION_distdir {
   my ($self) = @_;
 
-  my $metafile = 'META.yaml';
+  my $metafile = 'META.yml';
   $self->write_metadata($metafile);
 
   my $dist_dir = $self->dist_dir;
@@ -932,9 +923,10 @@ sub link_c {
   my $cf = $self->{config}; # For convenience
 
   my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$cf->{dlext}"));
-  unless ($self->up_to_date("$file_base$cf->{obj_ext}", $lib_file)) {
+  my $objects = $self->{objects} || [];
+
+  unless ($self->up_to_date("$file_base$cf->{obj_ext}", [$lib_file, @$objects])) {
     my $linker_flags = $cf->{extra_linker_flags} || '';
-    my $objects = $self->{objects} || [];
     $self->do_system("$cf->{shrpenv} $cf->{cc} $cf->{lddlflags} -o $lib_file ".
 		     "$file_base$cf->{obj_ext} @$objects $linker_flags")
       or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
